/* Intro {{{
 * ---------------------------------------------------------------- 
 *  DConnect Daemon 
 * 
 *	  #(@) Copyright (c) 2002, DConnect development team 
 *	  #(@) Homepage: http://www.dc.ds.pg.gda.pl/ 
 * 
 *  This  program is free software; you can redistribute it and/or 
 *  modify it under the terms of the GNU General Public License as 
 *  published by the Free Software Foundation; either version 2 of 
 *  the License. 
 * 
 *  This program is distributed in the hope that it will be useful, 
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the 
 *  GNU General Public License for more details. 
 * 
 *  You  should  have  received a copy  of  the GNU General Public 
 *  License  along  with  this  program;  if  not,  write  to  the 
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge, 
 *  MA 02139, USA. 
 * 
 * ---------------------------------------------------------------- 
 * }}} */ 
 
/* include local header files {{{ */ 
#include "pch.h" 
#include "main.h" 
#include "conf.h" 
#include "net.h" 
#include "misc.h" 
#include "match.h" 
#include "penalties.h" 
#include "cmd.user.h" 
/* }}} */ 
 

extern int NPENAL; /* penalties count */ 
extern int NUSERS; /* penalties count */ 
 
extern Tpenalty *penalties[MAXPENALTIES]; /* table of penalties */ 
extern struct userrec *user[MAXUSERS];	/* users table */ 
extern struct config conf;		/* configuration data */ 
 
 
/*  the assistant function {{{ */ 
 
/* cmd_nick2id(*usr,*nick) 
.description. 
			gets the id for the nick; if the nick is not specified or user does not exist 
			apropriate message is shown to the usr 
 
.args. 
			struct userrec *usr 	-IN-	 sending user's userec 
			char *nick				-IN-	 the nick, whose id we want 
	 
.return values. 
			<value> - the id of the user 
			-1	  - user not exists or nick not specified 
{{{ */ 
int cmd_nick2id(struct userrec *usr, char *nick) 
{ 
	int n; 
	 
	if (!nick) 
	{ 
		pubmsg(usr,"	User nick needed"); 
		pubmsg(usr,"========================================"); 
		return -1; 
	} 
	n=nick2id(nick); 
	 
	if (n<0) 
	{ 
		pubmsg(usr,"	User '%s' not found",nick); 
		pubmsg(usr,"========================================"); 
		return -1; 
	} 
	return n; 
} /* }}} */ 
	 

/* fmthln(*usr, *perm, console, visibility, *commnd, *description) 
.description. 
		displays formatted help line to >usr< whose usr->perm matches one of the >perm< 
		the line format is 
 		in chat: <HUB>#commmand		  <- decription 
  		in cons: commmand				<- decription 

.args. 
		struct userrec *usr - IN -	the user record; if usr is console user, does not display '#' before commnd 
  		char *perm			- IN - contains permissions i.e:"cas"; only users with one of thease will see the line 
		int  console		- IN - determines if line is visible from console, chat or both 
		char *commnd  		- IN - contains the command; if no specified, does not display '#' before commnd 
	 	char *description	- IN - contains description of the command; if no description is specified, display commnd as <HUB> commnd ; else display <HUB> (empty line) 

.exaple use. 
		fmthln(usr,"cas","Test",NULL);	---displays-->	'<HUB> Test' 
		fmthln(usr,"cas",NULL,"Test");	---displays-->	'					Test' 
		fmthln(usr,"cas","Axe","Test");	---displays-->	'#Axe				Test' 

{{{*/ 
void fmthln(struct userrec *usr,char *perm, int visibility, char *commnd, char *description) 
{ 
	if ((perm) && ( 
		!(strchr(usr->perm,'s') && strchr(perm,'s')) && 
		!(strchr(usr->perm,'c') && strchr(perm,'c')) && 
		!(strchr(usr->perm,'a') && strchr(perm,'a')) )) return; 

	if (!(usr->cons && visibility&VIS_CONS) && !(!(usr->cons) && visibility&VIS_CHAT)) return;
	
	//if user is console user or no commnd is specified, display '#' before commnd 
	if (description) pubmsg(usr,(commnd)?"%c%-30s<- %s":"%c%-30s %s" ,(usr->cons==0)?((!commnd)?' ':'#'):' ',(commnd)?commnd:"",description); 
	//if no description is specified, display commnd as pubmsg; else display empty line as pubmsg 
	else pubmsg(usr,"%s",(commnd)?commnd:""); 
}/* }}} */ 
	 
 
/* penalprvmsg(*to,*from,*fmt,...) 
.description. 
	sends the private  message to from 'from' userrec 'to' userrec; 
	the message contains type of penalty and reason; 
.args. 
	struct userrec *to	//recievers's userrec 
 	struct userrec *from 	//sender's userrec 
  	char *message		//the message 
{{{ */ 
void penalprvmsg(struct userrec *to, struct userrec *from, char *fmt, ...) 
{ 
	char *str=NULL;
	va_list args;
	va_start(args , fmt);

	str=(char *)make_message(fmt,args);
	
	privmsg(to,from,str); 

	va_end(args);

	if ((to->penalty) && (to->penalty->reason))
 	{
		privmsg(to,from,"R e a s o n:");
		privmsg(to,from,to->penalty->reason);
	}
	my_free(str);
	
} /* }}} */ 
 

/* opbreak - user with less permissions can not run a command on OP or sOP {{{ */ 
int opbreak(struct userrec *usr,struct userrec *op, char *commnd) 
{ 
	if(!commnd) 
	{ 
		pubmsg(usr,"ERROR: in 'opbrake': command not specified; returning 1; "); 
		return 1; 
	} 
 
	if (usr->perm!=NULL && op->perm!=NULL) 
	{ 
		if (strchr(op->perm,'a') && strchr(usr->perm,'a')) 
		{ 
			pubmsg(usr,"	You are trying to %s OP", commnd);
			pubmsg(usr,"========================================"); 
			return 1; 
		} 
 
		if (strchr(op->perm,'s') && (strchr(usr->perm,'a') || strchr(usr->perm,'s'))) 
		{ 
		pubmsg(usr,"	You are trying to %s sOP", commnd);
		pubmsg(usr,"========================================"); 
		pubmsg(op,"%s is trying to %s you.", usr->nick,commnd);
		return 1; 
		} 
 
		if (strchr(op->perm,'a') && strchr(usr->perm,'s')) 
		{ 
   		pubmsg(usr,NULL,"You are %sing OP", commnd);
		pubmsg(usr,"========================================");
		return 0; 
		} 
	} 
	 
	return 0; 
 
}/* }}} */ 
	 
/* }}} */ 
/* the COMMAND functions {{{ */ 
 
/* cmdhelp - shows usr the help screen {{{*/ 
void cmd_help(struct userrec *usr,char *line) 
{ 
	fmthln(usr,NULL,  VIS_ALL,	"========================================",	NULL); 
	fmthln(usr,NULL,  VIS_ALL,  "HELP",	NULL); 
	fmthln(usr,NULL,  VIS_ALL,  "========================================", NULL); 

	fmthln(usr,NULL,  VIS_ALL,	"[commands listed by categories]",	NULL); 
	fmthln(usr,NULL,  VIS_ALL,  NULL,NULL); 

//	fmthln(usr,NULL,  VIS_ALL,	"-----------------------------------------",NULL); 
//	fmthln(usr,NULL	, VIS_ALL,	"help",		"shows help (commands listed alphabeticly)"); 
//	fmthln(usr,NULL,  VIS_ALL,	"-----------------------------------------",NULL); 
//  fmthln(usr,NULL,  VIS_ALL,  NULL,NULL); 

	fmthln(usr,NULL,  VIS_ALL,	"[info]:",	NULL); 
	fmthln(usr,NULL,  VIS_ALL,	"-----------------------------------------", NULL); 
	fmthln(usr,"c",   VIS_CONS,	"conf",			"shows current hub configuration"); 
	fmthln(usr,"c",	  VIS_CONS,	"stat",			"shows statistics"); 
	fmthln(usr,NULL,  VIS_ALL,	"uptime",		"shows hub uptime"); 
	fmthln(usr,"c",   VIS_CONS,	"users",		"shows all hub users"); 
	fmthln(usr,NULL,  VIS_ALL,	"version",		"shows version of the hub"); 
	fmthln(usr,"c",   VIS_CONS,	"who",			"shows who is on the console"); 
	fmthln(usr,"sac", VIS_ALL,	"whois <nick>",	"shows user description"); 
	fmthln(usr,NULL,  VIS_ALL,	"-----------------------------------------", NULL); 
	fmthln(usr,"as",  VIS_ALL,  NULL,NULL); 

	fmthln(usr,"as",  VIS_ALL,	"[punishments]:", NULL); 
	fmthln(usr,"as",  VIS_ALL,	"-----------------------------------------",NULL); 
	fmthln(usr,"as",  VIS_ALL,	"amnesty <ip>",					"removes penalty from ip"); 
	fmthln(usr,"as",  VIS_ALL,	"amnesty_all",					"clears the punished ip's list"); 
	fmthln(usr,"as",  VIS_ALL,	"lspunished",					"shows the list of punished ip's"); 
	fmthln(usr,"as",  VIS_ALL,	"punish <nick> <how> [how long in minutes] [reason]","punishes the user");
	fmthln(usr,"as",  VIS_ALL,	NULL,							"<how>=(nosearch,nopblchat,noprvchat,nodl,ban)"); 
	fmthln(usr,"as",  VIS_ALL,	"kick <nick> <reason>", 		"disconnects the user"); 
	fmthln(usr,"as",  VIS_ALL,	"-----------------------------------------",NULL); 
	fmthln(usr,"as",  VIS_ALL,  NULL,NULL); 

	fmthln(usr,"as",  VIS_ALL,	"[communication & redir]:" ,NULL); 
	fmthln(usr,"as",  VIS_ALL,	"-----------------------------------------" ,NULL); 
	fmthln(usr,"as",  VIS_ALL,	"msg <nick> <message>"		,"sends message to user in pub chat as hub"); 
	fmthln(usr,"s",   VIS_ALL,	"msg_all <message>"			,"sends message to all users in pub chat as hub"); 
	fmthln(usr,"as",  VIS_ALL,	"redirect <nick> <where>"	,"moves user to another hub"); 
	fmthln(usr,"s",   VIS_ALL,	"redirect_all <where>"		,"moves all users to another hub"); 
	fmthln(usr,"sa",  VIS_ALL,  "-----------------------------------------" ,NULL); 
	fmthln(usr,"sa",  VIS_ALL,  NULL ,NULL); 
	
	fmthln(usr,"c",   VIS_CONS,	"[other commands]:" ,NULL); 
	fmthln(usr,"c",   VIS_CONS,	"-----------------------------------------" ,NULL); 
	fmthln(usr,"c",   VIS_CONS,	"rehash"	,"reloads hub configuration"); 
	fmthln(usr,"c",   VIS_CONS,	"quit"		,"leave the console"); 
	fmthln(usr,"c",   VIS_CONS,	"debug <n>"	,"sets debug level"); 
	fmthln(usr,"c",   VIS_CONS,	"jq"		,"toggles displaing of joins & quits traffic"); 
	fmthln(usr,"c",   VIS_CONS,	"boost"		,"toggles info-booster"); 
	fmthln(usr,"c",   VIS_CONS,	"-----------------------------------------" ,NULL); 
	return; 

} /* }}} */ 
 
/* hlp_redirect -  shows help for redirect command {{{*/ 
void hlp_redirect(struct userrec *usr) 
{ 
	pubmsg(usr," "); 
	pubmsg(usr,"Description: "); 
	pubmsg(usr,"	Moves the user to another hub."); 
	pubmsg(usr," "); 
	pubmsg(usr,"Syntax:"); 
	pubmsg(usr,"	%credirect <nick> <where>",(usr->cons)?' ':'#'); 
	pubmsg(usr," "); 
	pubmsg(usr,"	<nick> is the nick of user"); 
	pubmsg(usr,"	<where> is the adress of hub into <user> should be moved"); 
	pubmsg(usr,"========================================"); 
} /* }}} */ 
 
/* cmdredirect - redirect's the user to another hub {{{ */ 
void cmd_redirect(struct userrec *usr,char *line) 
{ 
	int n; 
	char *tmp1=NULL, *tmp2=NULL; 

	//checking if user has apriopriate permissions
	if (!strchr(usr->perm,'s') && !strchr(usr->perm,'a')) return;

	tmp1=strtok_fix(line," "); //who 
	tmp2=strtok_fix(NULL,"|");//where 
 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"   REDIRECT"); 
	pubmsg(usr,"========================================"); 
 
	if (!line) { hlp_redirect( usr );  return;} 
	 
	// checks the nick, returns id 
	n=cmd_nick2id(usr,tmp1); if(n<0)return; 
	if (!tmp2) 
	{ 
		pubmsg(usr,"	Do not know where to redirect"); 
		pubmsg(usr,"========================================"); 
		return; 
	} 
 
	if (!strcmp(usr->nick,tmp1)) 
	{ 
		pubmsg(usr,"	You are trying to move yourself"); 
		pubmsg(usr,"========================================"); 
		return; 
	} 
 
	if (opbreak(usr,user[n],"redirect"))return; 
		 
	debug(DEBUG_NET,"'%s'(OP) moved %s to '%s'",usr->nick,user[n]->nick,tmp2); 
 
	sendf(user[n],"$ForceMove %s|",tmp2); 
	pubmsg(usr,"	Redirecting '%s' to %s",tmp1,tmp2); 
	pubmsg(usr,"========================================"); 
 
	if((n=nick2id(tmp1))>=0) user_set_state(user[n],STATE_QUIT_SHOW); 
} /* }}} */ 
 
/* hlp_redirect -  shows help for redirect command {{{*/ 
void hlp_redirect_all(struct userrec *usr) 
{ 
	pubmsg(usr," "); 
	pubmsg(usr,"Description: "); 
	pubmsg(usr,"	Moves all the users to another hub."); 
	pubmsg(usr," "); 
	pubmsg(usr,"Syntax:"); 
	pubmsg(usr,"	%credirect_all <where>",(usr->cons)?' ':'#'); 
	pubmsg(usr," "); 
	pubmsg(usr,"	<where> is the adress of hub into users should be moved."); 
	pubmsg(usr,"========================================"); 
} /* }}} */ 
 
/* cmdredirectall - redirect all users to anther hub {{{*/ 
void cmd_redirect_all(struct userrec *usr,char *line) 
{ 
	int temp=0,n; 
	char *tmp1=NULL; 

	//checking if user has apriopriate permissions
	if (!strchr(usr->perm,'s')) return; 
	 
	tmp1=strtok_fix(line,"| "); //<where> 
	 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"   REDIRECT_ALL"); 
	pubmsg(usr,"========================================"); 
 
	if (!line) {hlp_redirect_all( usr );  return;} 
	if (!tmp1) 
	{ 
		pubmsg(usr,"	do not know where to redirect to"); 
	  	pubmsg(usr,"========================================"); 
		return; 
	} 
 
	debug(DEBUG_NET,"'%s'(sOP) moved ALL to '%s'",usr->nick,tmp1); 
 
	pubmsg(usr,"	Moving users to %s",tmp1); 
 
	temp=0; 
	for(n=0;n<NUSERS;n++) 
		if (user[n] && user_tst_state(user[n],STATE_LOGGEDIN|STATE_REGISTERED) && !strchr(user[n]->perm,'s')) 
		{ 
			temp++; 
			sendf(user[n],"$ForceMove %s|",tmp1); 
		} 
 
	pubmsg(usr,"	Communicate sent to %d of %d users",temp,NUSERS); 
	pubmsg(usr,"========================================"); 
	debug(DEBUG_NET,"'%s'(OP) moved ALL to '%s'",usr->nick,tmp1); 
} /* }}} */ 
 
/* cmdpassword {{{*/ 
void cmd_password(struct userrec *usr,char *line) 
{ 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"   PASSWORD"); 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"	Command not supported yet"); 
	pubmsg(usr,"========================================"); 
} /* }}} */ 
 
/* cmduptime - displays hub uptime {{{ */ 
void cmd_uptime(struct userrec *usr) 
{ 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"   UPTIME"); 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"	%s",uptime(starttime)); 
	pubmsg(usr,"========================================"); 
} /* }}} */ 
 
/* cmdversion - displays hub version {{{ */ 
void cmd_version(struct userrec *usr) 
{ 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"   VERSION"); 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"	%s (build: %s)",PACKAGE_VERSION,BUILD_TIME); 
	pubmsg(usr,"========================================"); 
} /* }}} */ 
 
 	/* cmdlspunished - displays the list of punished {{{*/ 
void cmd_lspunished(struct userrec *usr) 
{ 
	int n; 

	//checking if user has apriopriate permissions
	if (!strchr(usr->perm,'s') && !strchr(usr->perm,'a')) return; 
 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"   LSPUNISHED"); 
	pubmsg(usr,"========================================"); 
 
	if (NPENAL==0) pubmsg(usr,"	Noone was punished :("); 
	else pubmsg(usr,"	IP : Penalty : Time : Reason"); 
 
	for(n=0;n<NPENAL;n++) pubmsg(usr,"	%s : %d : %ld : %s",penalties[n]->ip,penalties[n]->penalty,user_exp_penalty(penalties[n]),penalties[n]->reason); 
	pubmsg(usr,"========================================"); 
 
} /* }}} */ 


/* hlp_punish - displays help for punish command {{{*/ 
void hlp_punish(struct userrec *usr) 
{ 
	pubmsg(usr,"	Punishment not specified"); 
	pubmsg(usr,"	Available punishments sample use:"); 
	pubmsg(usr," "); 
	pubmsg(usr,"	%cpunish <nick> <how> [time] [reason]",(usr->cons)?' ':'#');
	pubmsg(usr," "); 
	pubmsg(usr,"	<how>"); 
	pubmsg(usr,"	nosearch		- user has no permission to use search"); 
	pubmsg(usr,"	nopblchat		- user has no permission to chat in main chat"); 
	pubmsg(usr,"	noprvchat		- user has no permission to chat in private chat"); 
	pubmsg(usr,"	nochat			- user has no permission to	chan in main and private chat"); 
	pubmsg(usr,"	nodl			- user has no permission to download"); 
	pubmsg(usr,"	ban				- user is banned"); 
	pubmsg(usr," ");
	pubmsg(usr,"	You can punish with multiple punishments, merging them with '+'");
	pubmsg(usr,"	i.e. ");
	pubmsg(usr,"		%cpunish UserNick nochat+nodl+nosearch 20",(usr->cons)?' ':'#');
	pubmsg(usr," ");
	pubmsg(usr,"	[time]			- the time in minutes");
	pubmsg(usr," ");
	pubmsg(usr,"	[time] and [reason] are optional");
	pubmsg(usr," 	if no time is specified the duration of penalty will be %ld minute(s)",STD_PENALTY_DURATION);
	pubmsg(usr," ");
	pubmsg(usr,"	1 day - 1440 minutes");
	pubmsg(usr,"	3 days - 4320 minutes");
	pubmsg(usr,"	1 week - 10080 minutes");
	pubmsg(usr,"	eternity (1 year) - 0 minutes :)");
	pubmsg(usr," ");
	pubmsg(usr,"	examples:");
	pubmsg(usr,"		%cpunish UserNick nochat 10 Was naughty.",(usr->cons)?' ':'#');
	pubmsg(usr,"		%cpunish UserNick nochat+nodl",(usr->cons)?' ':'#');
	pubmsg(usr,"		%cpunish UserNick nosearch Was naughty.",(usr->cons)?' ':'#');
	pubmsg(usr,"========================================");
} /* }}} */ 
 
/* cmdpunish - punishes the user with one of the avaliable penalties {{{ */ 
void cmd_punish (struct userrec *usr,char *line) 
{ 
	const char *punish[]= 
	{ 
	"nopblchat", 
	"ban", 
	"nosearch", 
	"noprvchat", 
	"nodl", 
	"nochat", 
	"" /* indicates end of list */ 
	}; 
 
	long n=0,countpunish=0, duration=0;
	char *tmp1=NULL,*tmp2=NULL,*tmp3=NULL,*tmp4=NULL,*pun=NULL;
	int punishment=0;
 
	//checking if user has apriopriate permissions
	if (!strchr(usr->perm,'s') && !strchr(usr->perm,'a'))return; 

	if (!line)
	{ 
		hlp_punish(usr); 
		return;
	}

	tmp1=strtok_fix(line," "); //who 
	tmp2=strtok_fix(NULL," "); //how 
	tmp3=strtok_fix(NULL," "); //time in minutes or reason
	tmp4=strtok_fix(NULL,""); //reason
 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"   PUNISH"); 
	pubmsg(usr,"========================================"); 

	if (!tmp2)
	{
		hlp_punish(usr) ;
		return;
	} 
 
	n=cmd_nick2id(usr,tmp1);  
	if(n<0)return; 

	if (tmp3)
	{
	    duration=strtol(tmp3,NULL,0); //read the value of tmp3
		if (tmp3!="0") 
		{
		    if (duration==0) //tmp3 is the reason (tmp3 isn't value);
	     	{
		       	duration=STD_PENALTY_DURATION;
		       	tmp4=tmp3;
		  	}
	    	// else tmp3 is the time and the reason is NULL (tmp3 is value);
		} else duration=525600; //if tmp3 is "0" then the penalty will be 1 year long :)
	}
 	else duration = conf.std_penalty_duration; //if tmp3==NULL

 	duration=duration*60;// to get duration in seconds
	pun=strtok_fix(tmp2,"+ \n");

	do
	{
	/* parse punishment {{{ */
		countpunish=0; //must be zero because of 'countpuinish++' above
		while(*punish[countpunish])
		{
			if (!strcasecmp(pun,punish[countpunish])) break;
			else countpunish++;
		}

		switch(countpunish)
		{
			case 0: //nopblchat
					punishment|=PENALTY_NOPBLCHAT;
					user_set_penalty(user[n]->ip,punishment,duration,tmp4,0);
					pubmsg(NULL,"'%s' has no permission to talk in public chat for %ld seconds.",user[n]->nick,duration);
					penalprvmsg(user[n],usr,"You have no permission to talk in public chat for %ld seconds.",duration); 
					//DEBUGMSG 
					break; 
 
			case 1: //ban 
					punishment|=PENALTY_BANNED;
					user_set_penalty(user[n]->ip,punishment,duration,tmp4,0);
					pubmsg(NULL,"'%s' is banned for %ld seconds.",user[n]->nick,duration); 
					penalprvmsg(user[n],usr,"You are banned for %ld seconds.",duration); 
					user_set_state(user[n],STATE_QUIT_SHOW); 
					break;
 
			case 2: //nosearch 
					punishment|=PENALTY_NOSEARCH;
					user_set_penalty(user[n]->ip,punishment,duration,tmp4,0);
					pubmsg(NULL,"'%s' has no permission to search for %ld seconds.",user[n]->nick,duration); 
					penalprvmsg(user[n],usr,"You have no permission to search for %ld seconds.",duration); 
					//DEBUGMSG 
					break; 
 
			case 3: //noprvchat 
					punishment|=PENALTY_NOPRVCHAT;
					user_set_penalty(user[n]->ip,punishment,duration,tmp4,0);
					pubmsg(NULL,"'%s' has no permission to talk in private chat for %ld seconds.",user[n]->nick,duration); 
					penalprvmsg(user[n],usr,"You have no permission to talk in private chat for %ld seconds.",duration); 
					//DEBUGMSG 
					break; 
 
			case 4: //nodl 
					punishment|=PENALTY_NODL;
					user_set_penalty(user[n]->ip,punishment,duration,tmp4,0);
					pubmsg(NULL,"'%s' has no permission to download anything for %ld seconds.",user[n]->nick,duration); 
					penalprvmsg(user[n],usr,"You have no permission to download anything for %ld seconds",duration); 
					//DEBUGMSG
					break; 
 
			case 5: //nochat 
					punishment|=PENALTY_NOPRVCHAT|PENALTY_NOPBLCHAT;
					user_set_penalty(user[n]->ip,punishment,duration,tmp4,0);
					pubmsg(NULL,"'%s' has no permission to talk in pub and private chat for %ld seconds.",user[n]->nick,duration); 
					penalprvmsg(user[n],usr,"You have no permission to talk in pub and private chat for %ld seconds.",duration); 
					//DEBUGMSG 
					break; 
 
			default: 
					pubmsg(usr,"Unknown punishment '%s'. Type '#punish' for more help",pun);
					break;
		} 
/* end of parse punishment }}}*/

 		pun=strtok_fix(NULL,"+");//get the next punishment
		
	}while (pun); //if another punishment specified, parse it

	
	pubmsg(usr,"========================================");
} /* }}} */ 
 
 
 
/* hlp_msg -  shows help for msg command {{{*/ 
void hlp_msg(struct userrec *usr) 
{ 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"	MSG"); 
	pubmsg(usr,"========================================"); 
	pubmsg(usr," "); 
	pubmsg(usr,"Description: "); 
	pubmsg(usr,"	Shows the <message> to the <user> as 'Hub'"); 
	pubmsg(usr," "); 
	pubmsg(usr,"Syntax:"); 
	pubmsg(usr,"	%cmsg <nick> <message>",(usr->cons)?' ':'#'); 
	pubmsg(usr," "); 
	pubmsg(usr,"========================================"); 
} /* }}} */ 
 
	 
/* cmdmsg - sends message as hub to the user {{{*/ 
void cmd_msg(struct userrec *usr,char *line) 
{ 
	int n; 
	char *tmp1=NULL,*tmp2=NULL; 

	//checking if user has apriopriate permissions
	if (!strchr(usr->perm,'s') && !strchr(usr->perm,'a')) return; 
	if (!line) {hlp_msg(usr); return;} 
	tmp1=strtok_fix(line," "); //who 
	tmp2=strtok_fix(NULL,""); //message 
 
	n=cmd_nick2id(usr,tmp1); if(n<0)return; 
 
	if (!tmp2) 
	{ 
		pubmsg(usr,"========================================"); 
		pubmsg(usr,"	MSG"); 
		pubmsg(usr,"========================================"); 
		pubmsg(usr,"	You have to specify message"); 
		pubmsg(usr,"========================================"); 
		return; 
	} 
 
	pubmsg(user[n],tmp2); 
}/* }}} */ 
	 
/* hlp_msg_all -  shows help for msg command {{{*/ 
void hlp_msg_all(struct userrec *usr) 
{
	pubmsg(usr,"========================================");
	pubmsg(usr,"	MSG_ALL");
	pubmsg(usr,"========================================");
	pubmsg(usr," ");
	pubmsg(usr,"Description: ");
	pubmsg(usr,"	Shows the <message> to all users as 'Hub'");
	pubmsg(usr," ");
	pubmsg(usr,"Syntax:");
	pubmsg(usr,"	%cmsg_all <message>",(usr->cons)?' ':'#');
	pubmsg(usr," ");
	pubmsg(usr,"========================================");
} /* }}} */

/* cmdmsgall - sends message as hub to all users {{{*/ 
void cmd_msg_all(struct userrec *usr,char *line) 
{
	//checking if user has apriopriate permissions
	if (!strchr(usr->perm,'s')) return; 
	if (!line) {hlp_msg_all(usr); return;} 
 
	pubmsg(NULL,line); 
	//DEBUGMSG 
} /* }}} */ 
 
/* cmd_amnesty - cancels all of current user's penalties {{{ */ 
void cmd_amnesty (struct userrec *usr,char *line) 
{ 
	char *tmp1=NULL; 

	//checking if user has apriopriate permissions
	if (!strchr(usr->perm,'s') && !strchr(usr->perm,'a')) return; 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"	AMNESTY"); 
	pubmsg(usr,"========================================"); 
	tmp1=strtok_fix(line," "); //ip 
 
	if (!tmp1) 
	{ 
	pubmsg(usr,"	User ip needed"); 
	  	pubmsg(usr,"========================================");	 
	return; 
	} 
 
	user_set_penalty(tmp1,PENALTY_NONE,0,NULL,0); 
	pubmsg(usr,"	Amnesty of IP: %s",tmp1); 
	pubmsg(usr,"========================================"); 
	//DEBUGMSG 
} /* }}} */ 

 /* cmdamnestyall - cancels all of penalties {{{ */
void cmd_amnesty_all (struct userrec *usr) 
{
	int i;

	//checking if user has apriopriate permissions
	if (!strchr(usr->perm,'s')) return;
	
	pubmsg(usr,"========================================");
	pubmsg(usr,"	AMNESTY_ALL");
	pubmsg(usr,"========================================");
	
	for(i=0; i<NPENAL;i++) delpenalty(0,0);
	pubmsg(usr,"	Amnesty of all punished");
	pubmsg(usr,"========================================");
	//DEBUGMSG 	
} /* }}} */ 
 
	 
/* cmd_conf {{{ */ 
void cmd_conf (struct userrec *usr) 
{ 
	//can be used only from console
	if (!usr->cons) return; 

	pubmsg(usr,"========================================"); 
	pubmsg(usr,"	CONF"); 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"Config file: %s", conf.conf_main); 
	pubmsg(usr,"Welcome file: %s", conf.conf_welcome); 
	pubmsg(usr,"MOTD file: %s", conf.conf_motd); 
	pubmsg(usr,"Bans file: %s", conf.conf_banned); 
	pubmsg(usr,"Console users file: %s", conf.conf_cusers); 
	pubmsg(usr,"Console allow file: %s", conf.conf_callow); 
	pubmsg(usr,"Nicks allow file: %s", conf.conf_nallow); 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"Console default debug level: %u", conf.debug_level); 
	pubmsg(usr,"Log level: %u", conf.log_level); 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"Log files dir: %s", conf.log_dir); 
	pubmsg(usr,"Main log: %s", conf.log_main); 
	pubmsg(usr,"Network access log: %s", conf.log_net); 
	pubmsg(usr,"Protocol log: %s", conf.log_proto); 
	pubmsg(usr,"Warnings log: %s", conf.log_wrng); 
	pubmsg(usr,"Console log: %s", conf.log_con); 
	pubmsg(usr,"Public chat log: %s", conf.log_chat); 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"Main listening port: %u", conf.listen_port_main); 
	pubmsg(usr,"Console listening port: %u", conf.listen_port_cons); 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"UID: %u", conf.uid); 
	pubmsg(usr,"GID: %u", conf.gid); 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"Hub Name: %s", conf.hubname); 
	pubmsg(usr,"User limit: %u", conf.userlimit); 
	pubmsg(usr,"Allow broken keys: %d", conf.allow_broken_key); 
	pubmsg(usr,"Ping timeout: %u", conf.ping_timeout); 
	pubmsg(usr,"Registration timeout: %u", conf.register_timeout); 
	pubmsg(usr,"Minimum data shared: %llu MB", conf.minshare); 
	pubmsg(usr,"Minimum slots: %u", conf.minslots); 
	pubmsg(usr,"Allow chat: %d", conf.allow_chat); 
	pubmsg(usr,"Flags: %u", conf.flags); 
	pubmsg(usr,"========================================"); 
} /* }}} */

/* cmd_rehash {{{ */
void cmd_rehash(struct userrec *usr)
{
	//can be used only from console
	if (!usr->cons) return;

	//checking if user has apriopriate permissions
	if (!strchr(usr->perm,'c')) return;
	pubmsg(usr,"========================================");
	pubmsg(usr,"	REHASH");
	pubmsg(usr,"========================================");
	finish(SIGHUP);
} /* }}} */

/* cmd_stat {{{ */
void cmd_stat (struct userrec *usr)
{
	int tmp=0,x;

	//can be used only from console
	if (!usr->cons) return;

	for(x=0;x<NUSERS;x++) tmp+=user[x]->share;
	tmp/=1024*1024; x=0; // MB
	if (tmp>=1024)
	{
		tmp/=1024; x=1; // GB
		if (tmp>=1024) { tmp/=1024; x=2; } // TB
	}

	pubmsg(usr,"========================================");
	pubmsg(usr,"	STAT");
	pubmsg(usr,"========================================");
	pubmsg(usr,"	Users: %d",NUSERS);
	pubmsg(usr,"	Shared data: %llu%cB",tmp,(x==0)?'M':((x==1)?'G':'T'));
	pubmsg(usr,"	Uptime: %s",uptime(starttime));
	pubmsg(usr,"========================================");
} /* }}} */


/* cmd_whois {{{ */ 
void cmd_whois (struct userrec *usr,char *line) 
{ 
	int x,count; 
	long long tmp;

	//checking if user has apriopriate permissions
	if (usr->perm && !strchr(usr->perm,'c') && 
		!strchr(usr->perm,'s') &&
		!strchr(usr->perm,'a')) return; 

	x=cmd_nick2id(usr,line); 
		 
	if (x<0) return; 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"	WHOIS"); 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"Nick: %s",user[x]->nick); 
	pubmsg(usr,"IP: %s",user[x]->ip); 
	pubmsg(usr,"Mode: %s",(user[x]->flag==1)?"online":((user[x]->flag==3)?"away":"unknown")); 
	pubmsg(usr,"Idle: %s",uptime(user[x]->ping)); 

	if (user[x]->cons==0) 
	{ 
		tmp=user[x]->share; 
		tmp/=1024*1024; count=0; // MB 
		if (tmp>=1024) 
	{ 
		tmp/=1024; count=1; // GB 
		if (tmp>=1024) { tmp/=1024; count=2; } // TB 
	} 
	pubmsg(usr,"Shared: %llu%cB",tmp,(count==0)?'M':((count==1)?'G':'T')); 
	} 
 
	pubmsg(usr,"E-mail: %s",user[x]->email); 
	pubmsg(usr,"Connection: %s",user[x]->ctype); 
	pubmsg(usr,"Description: %s",user[x]->desc); 
	pubmsg(usr,"Permissions: %s",user[x]->perm); 
	 
	if (user[x]->penalty) pubmsg(usr,"Punishments: %d",user[x]->penalty->penalty); 
	pubmsg(usr,"========================================"); 
} /* }}} */ 
 
/* cmd_boost {{{ */ 
void cmd_boost (struct userrec *usr) 
{ 
	//can be used only from console
	if (!usr->cons) return; 

	pubmsg(usr,"========================================"); 
	pubmsg(usr,"	BOOST"); 
	pubmsg(usr,"========================================"); 

	toggle_flag(&conf,usr,INFO_BOOST); 
} /* }}} */ 
 
/* cmd_jq {{{ */ 
void cmd_jq (struct userrec *usr) 
{ 
	//can be used only from console
	if (!usr->cons) return; 

	pubmsg(usr,"========================================"); 
	pubmsg(usr,"	JQ"); 
	pubmsg(usr,"========================================"); 
 
	toggle_flag(&conf,usr,SHOW_JQ); 
} /* }}} */ 
 
/* cmd_quit {{{ */ 
void cmd_quit (struct userrec *usr) 
{ 

	//can be used only from console
	if (!usr->cons) return; 
	if (!strchr(usr->perm,'c')) return; 

	pubmsg(usr,"========================================"); 
	pubmsg(usr,"	QUIT"); 
	pubmsg(usr,"========================================"); 
	user_set_state(usr,STATE_QUIT_SHOW); 
} /* }}} */ 
 
/* cmd_debug {{{ */ 
void cmd_debug (struct userrec *usr,char *line) 
{ 
	int x; 

	//can be used only from console
	if (!usr->cons) return; 
  
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"DEBUG"); 
	pubmsg(usr,"========================================"); 
 
	if (!line) 
	{ 
	pubmsg(usr,"Current debug level: %d.",usr->debug); 
	pubmsg(usr,"========================================"); 
  	return; 
	} 
 
	x=atoi(line); 
	if (!*line) return; 
	if (x<1) x=1; 
	usr->debug=x; 
	pubmsg(usr,"Debug level set to: %d.",usr->debug); 
	pubmsg(usr,"========================================"); 
	} /* }}} */ 
 
/* cmd_who {{{ */ 
void cmd_who (struct userrec *usr) 
{
	int x;
	if (!usr->cons) return; 
	if (!strchr(usr->perm,'c')) return; 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"	WHO"); 
	pubmsg(usr,"========================================"); 

	pubmsg(usr,"USER				 FROM			 IDLE"); 
	for (x=0;x<NUSERS;x++) 
		if (user[x] && user_tst_state(user[x],STATE_REGISTERED) &&(user[x]->cons)) 
			sendf(usr,"%-20s %-15s  %s\r\n",user[x]->nick,user[x]->ip,uptime(user[x]->ping)); 
	pubmsg(usr,"========================================"); 
} /* }}} */ 
 
/* cmd_users {{{ */ 
void cmd_users (struct userrec *usr) 
{ 
	long i,tmp=0;
	 
	if (!usr->cons) return; 
 
	if (!strchr(usr->perm,'c')) return; 
 
	pubmsg(usr,"========================================"); 
	pubmsg(usr,"	USERS"); 
	pubmsg(usr,"========================================"); 
 
	for (i=0;i<NUSERS;i++) 
	{ 
		if (user[i] && user_tst_state(user[i],STATE_LOGGEDIN|STATE_REGISTERED))  
		{ 
			pubmsg(usr,"%-26s",user[i]->nick);  
			tmp=(tmp+1)%4; 
			
			if (!tmp) pubmsg(usr," ");  
		}
 
	}
	 
	if (tmp)  
	{
		pubmsg(usr," ");  
		tmp=0; 
	} 

	pubmsg(usr,"========================================"); 
} /* }}} */ 
 
 
/* cmd_kick {{{ */
void cmd_kick (struct userrec *usr,char *line)
{
	char *tmp1;
	int i;

	//checking if user has apriopriate permissions
	if (!strchr(usr->perm,'s') && !strchr(usr->perm,'a')) return;

	if (!line||!*line)  return;

	tmp1=strtok_fix(line," ");
	pubmsg(usr,"========================================");
	pubmsg(usr,"	KICK: %s",tmp1);
	pubmsg(usr,"========================================");

	i=cmd_nick2id(usr,tmp1); 
	
	if(i<0)return;

	if (user[i]==usr) 
	{
		pubmsg(usr,"	It's simplier to use 'quit'.");
		pubmsg(usr,"========================================");
	   	return;
	}

	if (opbreak(usr,user[i],"kick"))return;
		
	tmp1=strtok_fix(NULL,"");
	pubmsg(user[i],"You have been kicked off by %s.",usr->nick);
	if (tmp1) pubmsg(user[i],"Reason: %s.",tmp1);

	debug(DEBUG_STD,"Kicked off %s@%s by %s.",user[i]->nick,user[i]->ip,usr->nick);
	user_set_state(user[i],STATE_QUIT_SHOW);
	
} /* }}} */


/* 
void command(*usr,*line) 
 
.description: 
  runs the in main chat or console given command, depending on usr->cons 
 
.args: 
	struct userrec *usr  - userrec of the user, who gives the command 
	char *line - the line following the command 
	 
{{{*/ 
 
void command(struct userrec *usr,char *line) 
{ 
/* const decl. {{{ */ 
	const char *cmd[]= 
	{ 
		"#help",			//0 
		"#password", 
		"#uptime", 
		"#redirect", 
		"#redirect_all", 
		"#version",			//5 
		"#lspunished", 
		"#punish", 
		"#amnesty", 
		"#msg",				//10 
		"#msg_all", 
		"#amnesty_all", 
		"#ignore", 
		"#uignore", 
		"#lsignored",		//15 
		"#conf",			 
		"#rehash", 
		"#quit", 
		"#stat", 
		"#debug",			//20 
		"#whois",			 
		"#who", 
		"#users", 
		"#kick", 
		"#jq",				//25 
 		"#boost", 
 		"" /* indicates end of list */ 
	}; 
 
	const char *sep=" "; 
 /* }}} */ 
  
	int cmdcount=0; 
	char *cmdtmp=NULL; //stores the command 
	char *cmdparam=NULL; //stores the parameters 
	char *line_copy=NULL; //stores the copy of line 
 
/* parase the line {{{*/ 
	line_copy=strdup(line); 
 
	//get the command 
	cmdtmp=strtok_fix(line_copy,sep);  
	if (!usr->cons) cmdtmp=strtok_fix(NULL,sep);  
 
	//get the parameters of the command 
	cmdparam=strtok_fix(NULL,"\n"); 
 
	cmdcount=-1; 
	if (cmdtmp && (*cmdtmp=='#'||usr->cons) && *cmdtmp!=0) 
	{ 
		cmdcount=0; 
		while(*cmd[cmdcount]) 
		{ 
			if (!strcasecmp(cmdtmp,cmd[cmdcount]+usr->cons)) break; 
			else cmdcount++; 
		} 
		if (!usr->cons && *cmd[cmdcount]) sendf(usr,"%s|",line); 
	} 
 
/* to uncomment add // at beggining 
//DEBUG PURPOSES {{{ 
	pubmsg(usr,"{+ + + + + + + + + + + + + + + + + + + + + + + + + + + + }"); 
  	pubmsg(usr,"	cons: %d; perm: %s;",usr->cons,usr->perm); 
  	pubmsg(usr,"	line: '%s'",line); 
  	pubmsg(usr,"	command: '%s'; parameters:'%s'",cmdtmp,cmdparam); 
	pubmsg(usr,"{+ + + + + + + + + + + + + + + + + + + + + + + + + + + + }"); 
//DEBUG PURPOSES }}} */ 
 
/* }}} */ 
 
	switch (cmdcount) 
	{ 
		case CMD_HELP			:cmd_help(usr,cmdparam);		goto leave; 
		case CMD_PASSWORD		:cmd_password(usr,cmdparam); 	goto leave; 
		case CMD_UPTIME			:cmd_uptime(usr); 				goto leave; 
		case CMD_REDIRECT		:cmd_redirect(usr,cmdparam); 	goto leave; 
		case CMD_REDIRECT_ALL	:cmd_redirect_all(usr,cmdparam);goto leave; 
		case CMD_VERSION		:cmd_version(usr); 				goto leave; 
		case CMD_LSPUNISHED		:cmd_lspunished(usr);			goto leave; 
		case CMD_PUNISH			:cmd_punish(usr,cmdparam);		goto leave; 
		case CMD_AMNESTY 		:cmd_amnesty(usr,cmdparam);		goto leave; 
		case CMD_MSG			:cmd_msg(usr,cmdparam);			goto leave; 
		case CMD_MSG_ALL 		:cmd_msg_all(usr,cmdparam);		goto leave; 
		case CMD_AMNESTY_ALL 	:cmd_amnesty_all(usr);			goto leave; 
/*		case CMD_IGNORE			:cmd_ignore(usr,cmdparam);		goto leave; 
		case CMD_UIGNORE		:cmd_uignore(usr,cmdparam);		goto leave; 
		case CMD_LSIGNORED		:cmd_lsignored(usr,cmdparam);	goto leave; */ 
		case CMD_CONF			:cmd_conf(usr);					goto leave; 
		case CMD_REHASH			:cmd_rehash(usr);				goto leave; 
		case CMD_QUIT			:cmd_quit(usr);					goto leave; 
		case CMD_STAT			:cmd_stat(usr);					goto leave; 
		case CMD_DEBUG			:cmd_debug(usr,cmdparam);		goto leave; 
		case CMD_WHOIS			:cmd_whois(usr,cmdparam);		goto leave; 
		case CMD_WHO			:cmd_who(usr);					goto leave; 
		case CMD_USERS			:cmd_users(usr);				goto leave; 
		case CMD_KICK			:cmd_kick(usr,cmdparam);		goto leave; 
		case CMD_JQ				:cmd_jq(usr);					goto leave; 
		case CMD_BOOST			:cmd_boost(usr);				goto leave; 
	 
		default: 
			if(user_tst_penalty(usr,PENALTY_NOPBLCHAT) || usr->cons) goto leave; 
 
			if(conf.allow_chat) 
 			{ 
				if (cmdtmp && *cmdtmp=='#') sendf(usr,"%s|",line); 
				else sendf(NULL,"%s|",line); 
 
				debug(DEBUG_CHAT,"%s",line); 
			} 
			else pubmsg(usr,"Public chat is not allowed."); 
	} 
 
leave: 
	my_free(line_copy); 
} 
/*}}}*/ 
