/* $Id: main.h,v 1.72 2004/04/15 18:16:30 rav Exp $ */

/* Intro {{{
 * ---------------------------------------------------------------- 
 *  DConnect Daemon
 * 
 *      #(@) Copyright (c) 2002, DConnect development team
 *      #(@) Homepage: http://www.dc.ds.pg.gda.pl/
 *
 *  This  program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You  should  have  received a copy  of  the GNU General Public
 *  License  along  with  this  program;  if  not,  write  to  the
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge,
 *  MA 02139, USA. 
 *
 * ---------------------------------------------------------------- 
 * }}} */

#if !defined( __vXhz__MAIN_H__INCLUDED__ )
#define __vXhz__MAIN_H__INCLUDED__ 

#if !defined( BUILD_TIME )
#define BUILD_TIME "unknown"
#endif /* BUILD_TIME */

/* default configuration {{{ */

/* maximum sizes (for security)		*/
#define MAXBLOCK	100000
#define NICK_LEN	32
#define VER_LEN		32
#define DESC_LEN	256
#define CTYPE_LEN	64
#define EMAIL_LEN	256

/* main config file			*/
#define CONF_MAIN	"dcd.conf"

/* welcome message file			*/
#define CONF_WELCOME	"dcd.welcome"

/* Message Of The Day file		*/
#define CONF_MOTD	"dcd.motd"

/* bans file				*/
#define CONF_BANNED	"dcd.banned"

/* password file			*/
#define CONF_CUSERS	"console.users"

/* console host access file		*/
#define CONF_CALLOW	"console.allow"

/* allowed nicks file			*/
#define CONF_NALLOW	"nicks.allow"

/* a penalties file	*/
#define CONF_PENALTIES	"dcd.penalties"

/* main service listening port number	*/
#define LISTEN_PORT_MAIN 411

/* remote console listening port	*/
#define LISTEN_PORT_CONS 511

/* max number of simultanously connected clients (for static buffers)	*/
#define MAXUSERS	 5000


#define MAXPENALTIES 100*MAXUSERS
/* limit for number of users (dc protocol level); <=MAXUSERS (!!!)	*/
#define USERLIMIT	 1000

/* name of our hub, obviously		*/
#define HUBNAME	"DConnectDaemon"

/* set to 1, if hub should allow clients to connect even when they reply
 * with an invalid $Key string		*/
#define ALLOW_BROKEN_KEY 0

/* allow chats or not - default yes	*/
#define ALLOW_CHAT	 1

/* allow chats or not - default yes	*/
#define ALLOW_PASSIVE	 1

/* standard penalty duration in minutes */
#define STD_PENALTY_DURATION	10

/* ping timeout duration in seconds; 0 = no timeout	*/
#define PING_TIMEOUT 0

/* registration time limit in seconds; 0 = no limit	*/
#define REGISTER_TIMEOUT 30

/* console debug level		*/
#define DEBUG_LEVEL	3

/* log level			*/
#define LOG_LEVEL	31

/* main log file		*/
#define LOG_MAIN	"dcd.log"

/* network access log file	*/
#define LOG_NET		"access.log"

/* DC protocol stuff log file	*/
#define LOG_PROTO	"protocol.log"

/* internal errors & warnings log file	*/
#define LOG_WRNG	"warnings.log"

/* console log file		*/
#define LOG_CON		"console.log"

/* public chat log file		*/
#define LOG_CHAT	"chat.log"

/* end of default values }}} */

/* configuration flags {{{ */

#define SHOW_JQ		1
#define INFO_BOOST	2
//#define ?		4
//#define ?		8
//#define ?		16
//#define ?		32
//#define ?		64
//#define ?		128

/* end of configuration flags }}} */

/* debug / log flags {{{ */

#define DEBUG_SYS	1
#define DEBUG_STD	2
#define DEBUG_NET	4
#define DEBUG_PROTO	8
#define DEBUG_WRNG	16
#define DEBUG_CON	32
#define DEBUG_CHAT	64
//#define ?		128

/* end of debug / log flags }}} */

/* STATE / user flags {{{ */

#define STATE_QUIT			1 // quiting without notice
#define STATE_SHOW_QUIT		2 // this will be tested whether to sendinfo on quiting
#define STATE_QUIT_SHOW		3 // user will quit and send info

#define STATE_CONNECTED		4  // user is connected
#define STATE_KEY			8  // user sent valid key
#define STATE_PASSWORD		16 // user has to send password
#define STATE_LOGGEDIN		32 // user logged in
#define STATE_REGISTERED	64 // user send valid info

/* end of STATE / user flags }}} */


/* record with penalties data */
typedef struct
{
char *ip;
int penalty;
long start_date;	// start date of punishment
long end_date;  	// end date of punishment
char *reason;
} Tpenalty;


/* record with user's data */
struct userrec
{
 int sock;
 char *buf;  // uncomplete data holder
 char bufrd; // buffer ready flag
 
 char state; // STATE_*  
 
 char *nick;
 char *ip;
 char *ver;
 unsigned long int ping;
 unsigned long long int share;
 char flag;
 unsigned char  cons;//  0 - normal user
					//  1 - console user
 unsigned char debug;	// console debug level
 char *desc;
 char *ctype; 			// connection type
 char *email;
 char *perm;
 Tpenalty *penalty;			// penalty  ///
 char *password;
 
 /* i g n o r e */
 char **ignored; // list of ignored nicks
 int NIGNORED; // amount of ignored nicks
 /**/
};

/* filled with config file data */
struct config
{
 char *conf_main;
 char *conf_welcome;
 char *conf_motd;
 char *conf_banned;
 char *conf_cusers;
 char *conf_callow;
 char *conf_nallow;
 char *conf_penalties;
 unsigned char debug_level;
 unsigned char log_level;
 char *log_dir;
 char *log_main;
 char *log_net;
 char *log_proto;
 char *log_wrng;
 char *log_con;
 char *log_chat;
 unsigned short int listen_port_main;
 unsigned short int listen_port_cons;
 char *hubname;
 unsigned int userlimit;
 char allow_broken_key;
 char allow_chat;
 long std_penalty_duration;
 unsigned char ping_timeout;
 unsigned char register_timeout;
 char *nick_pattern;
 unsigned long long int minshare;
 unsigned int minslots;
 unsigned int allow_passive;
 unsigned int uid;
 unsigned int gid;
 unsigned char flags;
};

int user_set_state(struct userrec *usr,int state);
int user_tst_state(struct userrec *usr, int state);

void adduser( int sock, char *remote );
void deluser( struct userrec *usr, char *reason );
void finish( int sig );

#endif /* __vXhz__MAIN_H__INCLUDED__ */

/* VIM Settings {{{
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * soft-stop-width: 4
 * c indent on
 * End:
 * vim600: sw=4 ts=4 sts=4 cindent fdm=marker
 * vim<600: sw=4 ts=4
 * }}} */

