 
 
/* Intro {{{ 
 * ---------------------------------------------------------------- 
 *  DConnect Daemon 
 * 
 *	  #(@) Copyright (c) 2002, DConnect development team 
 *	  #(@) Homepage: http://www.dc.ds.pg.gda.pl/ 
 * 
 *  This  program is free software; you can redistribute it and/or 
 *  modify it under the terms of the GNU General Public License as 
 *  published by the Free Software Foundation; either version 2 of 
 *  the License. 
 * 
 *  This program is distributed in the hope that it will be useful, 
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the 
 *  GNU General Public License for more details. 
 * 
 *  You  should  have  received a copy  of  the GNU General Public 
 *  License  along  with  this  program;  if  not,  write  to  the 
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge, 
 *  MA 02139, USA. 
 * 
 * ---------------------------------------------------------------- 
 * }}} */ 
  
 /* include local header files {{{ */ 
#include "pch.h" 
#include "main.h" 
#include "match.h" 
#include "misc.h" 
#include "net.h" 
#include "penalties.h" 
#include "conf.h"

/* }}} */ 
 
extern int NPENAL; /* penalties count */ 
extern int NUSERS; /* penalties count */ 
 
extern Tpenalty *penalties[MAXPENALTIES]; /* table of penalties */ 
extern struct userrec *user[MAXUSERS];	/* users table */ 
 
extern pthread_mutex_t mutex_penalty;	/* provides synchronization between threads */ 
 
 
/* 
penalties_ip2itemnumber( *ip ) 
 
.description: 
	gets the penalties item number () containing the >ip< on the penalties list. 
	 
.args 
	char *ip		- IN  - the ip 
	 
.return values: 
	-1		- the ip is not present on the penalties list 
	value	- the number of the ithem containing the >ip< is >value< 
{{{*/ 
 
int penalties_ip2itemnumber(char *ip) 
{ 
	int itemnumber; 

	for(itemnumber=0;itemnumber<NPENAL;itemnumber++) 
		if (penalties[itemnumber] && match(penalties[itemnumber]->ip, ip)) return itemnumber; 

	return -1; 
} /* }}} */ 
 
 
/* 
void user_penalty(*usr) 
 
.description: 
  checks if the user->ip is listed on >penalties< list and changes user->penalty 
  to fit those from the list 
 
.args: 
	struct userrec *usr - IN(ip) & OUT(penalty) - userrec 
{{{*/ 
 
void user_penalty(struct userrec *usr) 
{ 
 
	int itemnumber; 
 
	itemnumber=penalties_ip2itemnumber(usr->ip); 

	if (itemnumber==-1) 
 	{ 
		usr->penalty=NULL; 
		return; 
	} 

 	usr->penalty=penalties[itemnumber]; 
 
	return; 
} /* }}} */ 
 
 
/* 
void addpenalty(*penrec, only_memory) 
 
.description: 
  adds the peanlty to penalties list and userrecs 
 
.args: 
	Tpenalty *penrec  - penalty record 
	int only_memory - if >0 then the penalties won't be written to the file conf.penalties
{{{*/ 
 
void addpenalty( Tpenalty *penrec, int only_memory)
{ 
	int i; 
 
	/* FIXME: find penalty wich ends in shortest amount of time */ 
	if (NPENAL+1>MAXPENALTIES) 
 	{ 
		my_free(penrec->ip); 
  		my_free(penrec->reason); 
  		my_free(penrec); 
  		return; 
	} 

/*	for (i=0;i<NUSERS;i++) 
		if (user[i] && match(user[i]->ip,penrec->ip) && strchr(user[i]->perm,'s')) return;*/
		
	pthread_mutex_lock( &mutex_penalty ); 
	penalties[NPENAL++]=penrec; /* change the item when it exists */ 
	pthread_mutex_unlock( &mutex_penalty ); 
 
 	/*add the item to the penalties list add the penalties to userrec*/ 
	for (i=0;i<NUSERS;i++) 
		if (user[i] && !strchr(user[i]->perm,'s') && match(user[i]->ip,penrec->ip)) user[i]->penalty=penrec; 

 	if (!only_memory) penalties_write();
}/* }}} */ 
 
 
/* 
void delpenalty(no, only_memory) 
 
.description: 
  removes the peanlty to penalties list and userrecs by itemnuber in penalties[] 
 
.args: 
	int no  - itemnumber of user's penalty in penalties list 
	int only_memory - if >0 then is removing penalties only from memory
							the conf.penalties file won't be written
{{{*/ 
 
void delpenalty( int no, int only_memory)
{ 
	int i; 
	Tpenalty *penrec=penalties[no]; 
 
 	/* remove penalty from userrecs */ 
	for (i=0;i<NUSERS;i++) 
		if (user[i] && user[i]->penalty==penrec) user[i]->penalty = NULL; 
 
	/* remove penalty from penalties[] */ 
	pthread_mutex_lock(&mutex_penalty); 
	penalties[no]=penalties[--NPENAL]; 
	penalties[NPENAL]=NULL; 
	pthread_mutex_unlock(&mutex_penalty); 
 
	/* free penrec */ 
	my_free(penrec->ip); 
  	my_free(penrec->reason); 
  	my_free(penrec); 

 	if (!only_memory) penalties_write();

}/* }}} */ 
 
 
/* 
user_set_penalty(*ip, penalty, duration, *reason) 
 
//duration=0 means "forever" (until hub reset) 
 
.description: 
	sets user's penalty 
 
.args 
	char *ip			- IN	- user's ip 
	int penalty			- IN	- the penalty {0-none} 
	long duration		- IN	- duration of the penalty {in minutes?) 
	char *reason 		- IN	- reason of penalty 
	int only_memory		- IN	- if we want to update conf.penalties file
 
{{{*/ 
 
void user_set_penalty(char *ip, int penalty, long duration, char *rsn, int only_memory)
{ 
	int itemnumber;		/* penalty list itemnumber */ 
	Tpenalty *penrec;   /* penalty record (used to add new penalty) */ 
	char *reason; 

	debug(DEBUG_PROTO,"user_set_penalty() enter"); 
 	itemnumber=penalties_ip2itemnumber(ip); 
 
	/* removing penalty (when penalty==PENALTY_NONE) {{{ */ 
	 
 	if (penalty==PENALTY_NONE) 
  	{ 
		if (itemnumber==-1) goto leave; ;			// the user's ip is not listed in the penalties list 
 		delpenalty(itemnumber,only_memory);	//delete the item conataining the user's ip, from the penalties list 
  		goto leave; 
	} /* }}} */ 
 
	/* if no reason */ 
	if (!rsn) reason=strdup("NO REASON"); 
	else reason=strdup(rsn); 
 
	/* adding a penalty (when penalty!=PENALTY_NONE) {{{ */ 
	 
   	if (itemnumber==-1) /* if penalety does not yet exist */ 
	{ 
 		penrec=(Tpenalty *) my_malloc(sizeof(Tpenalty));	/* create a penalty record... */ 
	   	penrec->ip=strdup(ip); 
		penrec->penalty=penalty; 


		penrec->reason=reason; 

		addpenalty(penrec,only_memory);	/* ...to add it */
		
		goto leave; 
	}

/* penalties[itemnumber]->penalty|= penalty; if penalety does exist |= new penalty */
	penalties[itemnumber]->penalty = penalty;	/* penalty overwrites existing */

	my_free(penalties[itemnumber]->reason); 
	penalties[itemnumber]->reason=reason;	/* set the new reason of penalty }}} */ 

	penalties[itemnumber]->start_date=(long)time(NULL); 
	penalties[itemnumber]->end_date=duration+penalties[itemnumber]->start_date; 

	if (!only_memory) penalties_write(); // updating conf.penalties file

leave:
	debug(DEBUG_PROTO,"user_set_penalty() leave"); 
} /* }}} */ 
 
 
/* 
user_exp_penalty(penalty) 
 
.description: 
	tests if penalty expired 
 
.args 
	Tpenalty *penalty	
 
.return values: 
	<=0: the >penalty< expired 
	>0: the >penalty< duration 
{{{*/ 
 
long user_exp_penalty(Tpenalty *penalty) 
{ 
  	return (penalty->end_date-(long)time(NULL)); 
} /* }}} */ 
 

/* 
user_tst_penalty(usr, penalty, duration) 
 
.description: 
	tests user's for penalties 
 
.args 
	struct userrec *usr - IN - userrec 
	int penalty			- IN - the penalty (0 - none) 
 
.return values: 
	0: the >penalty< does NOT match user's penalty, the user has no penalty; 
	1: the >penalty< DOES match user's penalty 
{{{*/ 
 
int user_tst_penalty(struct userrec *usr, int penalty) 
{ 

	//user was not punished
 	if (!(usr->penalty))return 0; 

	// if penalty expired
	if (user_exp_penalty(usr->penalty)<=0)
	{
		//remove user's penalty
		user_set_penalty(usr->ip,PENALTY_NONE,0,NULL,0);
		return 0;
	}

  	return (usr->penalty->penalty&penalty); 
} /* }}} */ 
 

